@@ -15,94 +15,27 @@ import signal
 import os
 import sys
 import tempfile
-import pyutilib.services
-from pyutilib.common import *
+import subprocess
 import copy
 from threading import Thread
-
-if sys.version_info[0:2] < (2,4): #pragma:nocover
-    print "WARNING: EXACT requires Python 2.4 or newer for robust process control."
-    if (sys.platform[0:3] == "win"):
-        raise OSError, "ERROR:   Aborting since we're on windows."
-    print "WARNING: Using a dummy subprocess mechanism that does not support timelimits."
-    import commands
-else:
-    import subprocess
-
-
-#
-# Note: the DummySubprocess.stdout file handle is closed the next time
-# that Popen is called.  This occurs because Popen returns a pointer to 'self'.
-#
-class DummySubprocess:
-    def __init__(self):
-        """
-        Initialize this object to have data like a subprocess module.
-        """
-        self.PIPE="pipe"
-        self.STDOUT="stdout"
-        self.mswindows = (sys.platform[0:3] == "win") or (sys.platform[0:4] == 'java')
-        self.tempfile = None
-        self.stdout = None
-
-    def __del__(self):
-        """
-        Cleanup temporary file descriptor and delete that file.
-        """
-        self.close()
-
-    def close(self):
-        """
-        Cleanup temporary file descriptor and delete that file.
-        """
-        if self.stdout is not None:
-            self.stdout.close()
-        if self.tempfile is not None:
-            os.unlink(self.tempfile)
-
-    def Popen(self, cmd,shell,stdout,stderr,preexec_fn):
-        """
-        Open a pipe'd stream, and setup self.stdout to be a file descriptor
-        for the output pipe.
-        """
-        if self.tempfile is not None:
-            self.close()
-        if stdout != "pipe":
-            raise IOError, "BUG: need to support other types of stdout"
-        (fhandle, self.tempfile) = tempfile.mkstemp(prefix="exact_",dir="/tmp")
-        os.close(fhandle)
-        (self.status,self.output) = commands.getstatusoutput(cmd.strip() + " > " + self.tempfile + " 2>&1")
-        self.stdout = open(self.tempfile,"r")
-        return self
-
-    def wait(self, timelimit=None):
-        """
-        We're using the commands module, so nothing needs to be done in wait().
-        """
-        return self.status
-
-    def poll(self):
-        """
-        Return the status returned by getstatusoutput().
-        """
-        return self.status
-
-
-if sys.version_info[0:2] < (2,4):
-    subprocess = DummySubprocess()
-
-if sys.version_info[0:2] >= (2,5) and subprocess.mswindows:
+if subprocess.mswindows and sys.version_info[0:2] < (2,5):
     import ctypes
 
-def kill_process(pid, sig=signal.SIGTERM, verbose=False):
+import pyutilib.services
+from pyutilib.common import *
+
+
+
+def kill_process(process, sig=signal.SIGTERM, verbose=False):
     """
     Kill a process given a process ID
     """
+    pid = process.pid
     if GlobalData.debug or verbose:
         print "Killing process",pid,"with signal",sig
     if subprocess.mswindows:
         if sys.version_info[0:2] < (2,5):
-            os.system("taskkill /t /f /pid "+`pid`)
+            os.system("taskkill /t /f /pid "+repr(pid))
         else:
             PROCESS_TERMINATE = 1
             handle = ctypes.windll.kernel32.OpenProcess(PROCESS_TERMINATE, False, pid)
@@ -154,7 +87,7 @@ def signal_handler(signum, frame, verbose=False):
        GlobalData.current_process.pid is not None and\
        GlobalData.current_process.poll() is None:
         GlobalData.signal_handler_busy=True
-        kill_process(GlobalData.current_process.pid, signum)
+        kill_process(GlobalData.current_process, signum)
         if verbose:
             print "  Signaled process", GlobalData.current_process.pid,"with signal",signum
         endtime = timer()+1.0
@@ -169,11 +102,11 @@ def signal_handler(signum, frame, verbose=False):
             GlobalData.signal_handler_busy=False
             if verbose:
                 print "Done."
-            raise OSError, "Interrupted by signal " + `signum`
+            raise OSError, "Interrupted by signal " + repr(signum)
         else:
-            raise OSError, "Problem terminating process" + `GlobalData.current_process.pid`
+            raise OSError, "Problem terminating process" + repr(GlobalData.current_process.pid)
         GlobalData.current_process = None
-    raise OSError, "Interrupted by signal " + `signum`
+    raise OSError, "Interrupted by signal " + repr(signum)
 
 
 #
@@ -218,7 +151,7 @@ def X_read_data(stream, ostr1, ostr2):
 # After this is finished, we can get the output from this command from
 # the process.stdout file descriptor.
 #
-def run_command(cmd, outfile=None, cwd=None, ostream=None, stdout=None, stderr=None, valgrind=False, valgrind_log=None, valgrind_options=None, memmon=False, env=None, define_signal_handlers=True, verbose=True, timelimit=None, tee=None, ignore_output=False, shell=True, thread_reader=_read_data):
+def run_command(cmd, outfile=None, cwd=None, ostream=None, stdout=None, stderr=None, valgrind=False, valgrind_log=None, valgrind_options=None, memmon=False, env=None, define_signal_handlers=True, debug=False, verbose=True, timelimit=None, tee=None, ignore_output=False, shell=True, thread_reader=_read_data):
     #
     # Move to the specified working directory
     #
@@ -295,6 +228,8 @@ def run_command(cmd, outfile=None, cwd=None, ostream=None, stdout=None, stderr=N
                 signal.signal(signal.SIGHUP, signal_handler)
             signal.signal(signal.SIGTERM, signal_handler)
     rc = -1
+    if debug:
+        print "Executing command '%s'" % cmd
     try:
         GlobalData.signal_handler_busy=False
         if (tee is None) or (tee is False):
@@ -483,7 +418,7 @@ class SubprocessMngr(object):
                 if status is not None:
                     return status
                 else:
-                    raise OSError, "Could not kill process " + `self.process.pid`
+                    raise OSError, "Could not kill process " + repr(self.process.pid)
 
     def stdout(self):
         return self.process.stdout
@@ -498,7 +433,7 @@ class SubprocessMngr(object):
         """
         Kill the subprocess and its children
         """
-        kill_process(self.process.pid, sig)
+        kill_process(self.process, sig)
         del self.process
         self.process = None
 
@@ -509,7 +444,7 @@ if __name__ == "__main__": #pragma:nocover
     stime = timer()
     foo = run_command("./dummy", tee=True, timelimit=10)
     print "A"
-    print "Ran for " + `timer()-stime` + " seconds"
+    print "Ran for " + repr(timer()-stime) + " seconds"
     print foo
     sys.exit(0)
 
@@ -551,8 +486,8 @@ if __name__ == "__main__": #pragma:nocover
     stime = timer()
     foo = run_command("python -c \"while True: pass\"", timelimit=10)
     print "A"
-    print "Ran for " + `timer()-stime` + " seconds"
+    print "Ran for " + repr(timer()-stime) + " seconds"
 
 
-pyutilib.services.register_executable(name="valgrind")
-pyutilib.services.register_executable(name="memmon")
+pyutilib.services.register_executable("valgrind")
+pyutilib.services.register_executable("memmon")
